//Douglas Fjällrud, Axel Blomén
package src.rsa;

import java.math.BigInteger;
import java.security.SecureRandom;

public class PrimeGenerator {
    private static final SecureRandom random = new SecureRandom();

    public static boolean isPrime(BigInteger n, int k) {
        if (n.equals(BigInteger.ONE) || n.mod(BigInteger.TWO).equals(BigInteger.ZERO)) {
            return false;
        }
    
        BigInteger s = n.subtract(BigInteger.ONE);
        int r = 0;
        while (s.mod(BigInteger.TWO).equals(BigInteger.ZERO)) {
            s = s.divide(BigInteger.TWO);
            r++;
        }
    
        for (int i = 0; i < k; i++) {
            BigInteger a = uniformRandom(BigInteger.TWO, n.subtract(BigInteger.ONE)); 
            BigInteger x = a.modPow(s, n);
    
            if (x.equals(BigInteger.ONE) || x.equals(n.subtract(BigInteger.ONE))) {
                continue; 
            }
    
            boolean isComposite = true;
            for (int j = 0; j < r - 1; j++) {
                x = x.modPow(BigInteger.TWO, n);
                if (x.equals(n.subtract(BigInteger.ONE))) {
                    isComposite = false;
                    break;
                }
            }
    
            if (isComposite) {
                return false;
            }
        }
    
        return true;
    }
    
    private static BigInteger uniformRandom(BigInteger min, BigInteger max) {
        BigInteger result;
        do {
            result = new BigInteger(max.bitLength(), random);
        } while (result.compareTo(min) < 0 || result.compareTo(max) > 0);
        return result;
    }

    public static BigInteger generatePrime(int bitLength) {
        BigInteger prime;
        do {
            prime = new BigInteger(bitLength, random).setBit(0);
        } while (!isPrime(prime, 20)); 
        return prime;
    }

    public static void benchmarkPrimes(int bitLength) {
        long startTime = System.nanoTime();
    
        for (int i = 0; i < 100; i++) {
            generatePrime(bitLength);
        }
    
        long endTime = System.nanoTime();
        double elapsedTime = (endTime - startTime) / 1_000_000_000.0; 
        System.out.println("Bitlängd: " + bitLength + " | Tid: " + elapsedTime + " sekunder");
    }

    public static void main(String[] args) {
        System.out.println("=== Benchmark: Primtalsgenerering ===");
        System.out.println("| Bitlängd | Antal | Tid (ms) |");
        System.out.println("|----------|-------|----------|");
    
        int[] bitSizes = {512, 1024, 2048, 4096}; 
        for (int bitSize : bitSizes) {
            long startTime = System.currentTimeMillis();
    
            for (int i = 0; i < 100; i++) {
                generatePrime(bitSize);
            }
    
            long endTime = System.currentTimeMillis();
            long elapsedTime = endTime - startTime; 
    
            System.out.printf("| %8d | %4d  | %8d |\n", bitSize, 100, elapsedTime);
        }
    
        System.out.println("\nBenchmark färdig! 🎉");
    }
}
